;;------------------------------------------------LICENSE NOTICE------------------------------------------------------------------------------
;;  This file is part of Amstrad CPC videogame "Zombie Disco"
;;  Copyright (C) 2022 PandemixStudio / Angel Robles Gonzalez/ David Costa Blanco / Christian Jose Mira Castello
;;  Copyright (C) 2020 ronaldo / Fremos / Cheesetea / ByteRealms (@FranGallegoBR)
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;------------------------------------------------LICENSE NOTICE-------------------------------------------------------------------------
.include "man/entity.h.s"
.include "man/game.h.s"
.include "sys/ai.h.s"

sys_ai_move_to::
    push ix
    pop iy

    xor a
    call man_entity_get
    ;enemy entity -> iy
    ;player entity -> ix
    ld c, #0
    ld a, e_x(ix)
    sub e_x(iy)
    jr nc, _posx_greater_or_equal
    _posx_lesser:
        ld e_vx(iy), #-1
        jr _endif_x
    _posx_greater_or_equal:
        jr z, _arrived_x
        ld e_vx(iy), #1
        jr _endif_x 
    _arrived_x:
        ld e_vx(iy), #0
        ld c, #1
    _endif_x:

    ld a, e_y(ix)
    sub e_y(iy)
    jr nc, _posy_greater_or_equal
    _posy_lesser:
        ld e_vx(iy), #-2
        jr _endif_y
    _posy_greater_or_equal:
        jr z, _arrived_y
        ld e_vy(iy), #2
        jr _endif_y 
    _arrived_y:
        ld e_vy(iy), #0
        ld a, c
        or a
        jr nz, _endif_y
            xor a
            ld e_vx(iy), a
            ld e_vy(iy), a
    _endif_y:
ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Ia 
;;Input: ix: entidad
sys_ai_left_right::
    ld a, e_x(ix)
    cp #0
    jr nz, no_izq
        ld e_vx(ix), #1
    no_izq:
        ld a, e_x(ix)
        push af
        ld a, #80
        sub e_w(ix)
        ld b, a
        pop af
        cp b
        jr nz, end_if
            ld e_vx(ix), #-1
    end_if:
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Ia 
;;Movimiento rebote
;;Input: ix: entidad
sys_ai_twist::
    ld a, e_x(ix)
    sub #B_LEFT
    jr nc, tw_no_izq
        ;Tope izquierda
        ;Moverse derecha
        ld e_vx(ix), #1
    tw_no_izq:
        ;No tope izquierda
        ld a, e_x(ix)
        sub #B_RIGHT
        add e_w(ix)
        jr nc, tw_end_if
            ;Tope derecha
            ;Moverse izquierda
            ld e_vx(ix), #-1
    tw_end_if:

    ld a, e_y(ix)
    sub #B_UP
    jr nc, tw_no_arr
        ;Tope arriba
        ;Moverse abajo
        ld e_vy(ix), #1
    tw_no_arr:
        ;No tope arriba
        ld a, e_y(ix)
        sub #B_DOWN
        add e_h(ix)
        jr nc, tw_end_if2
            ;Tope abajo
            ;Moverse arriba
            ld e_vy(ix), #-1
    tw_end_if2:
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Ia 
;;Movimiento rebote con contador de muerte
;;Input: ix: entidad
sys_ai_twist_time::
    call sys_ai_twist
    call sys_ai_destroy_time
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Ia
;;Girar en forma de cuadrado (der-abj-izq-arr)
;;Input: ix: entidad
sys_ai_box::
    ld a, e_x(ix)
    sub #B_LEFT
    jr nc, box_no_izq
        ;Tope izquierda
        ld a, e_y(ix)
        sub #B_DOWN
        add e_h(ix)
        jr nc, box_no_abj
            ;Tope abajo
            ;Moverse arriba
            ld e_vx(ix), #0
            ld e_vy(ix), #-1
        box_no_abj:
            ;No tope abajo
            ld a, e_y(ix)
            sub #B_UP
            jr nc, box_end_if
                ;Tope arriba
                ;Moverse derecha
                ld e_vx(ix), #1
                ld e_vy(ix), #0
    box_no_izq:
        ;No tope izquierda
        ld a, e_x(ix)
        sub #B_RIGHT
        add e_w(ix)
        jr nc, box_no_der
            ;Tope derecha
            ;Moverse abajo
            ld e_vx(ix), #0
            ld e_vy(ix), #1
        box_no_der:
            ;No tope derecha
            ld a, e_y(ix)
            sub #B_DOWN
            add e_h(ix)
            jr nc, box_end_if
                ;Tope abajo
                ;Moverse izquierda
                ld e_vx(ix), #-1
                ld e_vy(ix), #0
    box_end_if:

ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Ia 
;;Movimiento en una columna
;;Input: ix: entidad
sys_ai_col::
    ld a, e_y(ix)
    sub #B_UP
    jr nc, col_noarriba
        ;Tope arriba
        ;Moverse abajo
        ld e_vy(ix), #1
    col_noarriba:
    ld a, e_y(ix)
    sub #B_DOWN
    add e_h(ix)
    jr nc, col_noabajo
        ;Tope abajo
        ;Moverse arriba
        ld e_vy(ix), #-1
    col_noabajo:

ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Ia 
;;Movimiento en una fila
;;Input: ix: entidad
sys_ai_row::
    ld a, e_x(ix)
    sub #B_LEFT
    jr nc, row_noizq
        ;Tope izquierda
        ;Moverse derecha
        ld e_vx(ix), #1
    row_noizq:
    ld a, e_x(ix)
    sub #B_RIGHT
    add e_w(ix)
    jr nc, row_noder
        ;Tope derecha
        ;Moverse izquierda
        ld e_vx(ix), #-1
    row_noder:
ret

siCounter: .db #0

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Ia SI
;;Movimiento de fila a fila
;;Input: ix: entidad
sys_ai_si::
    ld a, e_x(ix)
    sub #B_LEFT
    jr nc, si_noizq
        ;Tope izquierda
        ld a, e_y(ix)
        sub #B_UP
        jr nc, si_noarriba
            ;Tope arriba
            ;Moverse abajo
            ld e_vx(ix), #0
            ld e_vy(ix), #1
            jp si_end
        si_noarriba:
        ;No tope arriba
        ld a, e_y(ix)
        sub #B_DOWN
        add e_h(ix)
        jr nc, si_noabajo
            ;Tope abajo
            ;Moverse derecha
            ld e_vx(ix), #1
            ld e_vy(ix), #0
            jp si_end
        si_noabajo:
        ;Ni tope arriba ni abajo
        call sys_ai_si_counter_add
        ld  a, (siCounter)
        sub #SI_COUNT
        jr nc, si_izqvertical
            ;Velocidad en x
            ld e_vx(ix), #0

            ld a,e_aiCounter(ix)
            ld e_vy(ix), a

            jp si_end
        si_izqvertical:
            ld e_vx(ix), #1
            ld e_vy(ix), #0
            jp si_end
    si_noizq:
    ld a, e_x(ix)
    sub #B_RIGHT
    add e_w(ix)
    jr nc, si_noder
        ;Tope derecha
        ld a, e_y(ix)
        sub #B_UP
        jr nc, si_noarriba2
            ;Tope arriba
            ;Moverse abajo
            ld e_vx(ix), #0
            ld e_vy(ix), #1

            ld e_aiCounter(ix), #1

            jp si_end
        si_noarriba2:
        ;No tope arriba
        ld a, e_y(ix)
        sub #B_DOWN
        add e_h(ix)
        jr nc, si_noabajo2
            ;Tope abajo
            ;ld e_vx(ix), #-1
            ;ld e_vy(ix), #0
            ;Moverse arriba
            ld e_vx(ix), #0
            ld e_vy(ix), #-1

            ld e_aiCounter(ix), #-1

            jp si_end
        si_noabajo2:
        ;Ni tope arriba ni abajo
        call sys_ai_si_counter_add
        ld  a, (siCounter)
        sub #SI_COUNT
        jr nc, si_izqvertical2
            ;Velocidad en x
            ld e_vx(ix), #0
            
            ld a,e_aiCounter(ix)
            ld e_vy(ix), a

            jp si_end
        si_izqvertical2:
            ld e_vx(ix), #-1
            ld e_vy(ix), #0
        jp si_end
    si_noder:
    si_end:
    call sys_ai_si_counter_restart
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Ia check SI counter add
;;
sys_ai_si_counter_add:
    ld  a, (siCounter)
    cp #SI_COUNT
    jr z,si_cd_max
        ;;Increase si counter
        ld a, (siCounter)
        inc a
        ld (siCounter), a
    si_cd_max:
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Ia check SI counter restart
;;
sys_ai_si_counter_restart:
    ld  a, (siCounter)
    cp #SI_COUNT
    jr nz,si_cd_no_max
        ;;Restart si counter
        ld a, #0
        ld (siCounter), a
    si_cd_no_max:
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Ia del enemigo
;;Input: ix: entidad
; sys_ai_nave_enemiga:
;     ld e_x(ix)
; ret

sys_ai_destroy_time::
    ld a, e_aiCounter(ix)
    dec a
    ld e_aiCounter(ix), a
    cp #0
    jr nz, no_destroy
        push ix
        pop hl
        call man_game_entity_destroy
        call man_game_enemy_dead
    no_destroy:
ret


sys_ai_update_one_entity::
    ld l, e_ai_funct(ix)
    ld h, e_ai_funct+1(ix)
    ld (ptrFunc+1), hl
    ptrFunc:
        call end_a
    end_a:
    ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Update ai
;;  Actualiza la ai de las entidades
sys_ai_update::
    ld hl, #sys_ai_update_one_entity
    ld d, #(e_cmp_ai | e_cmp_movable)
    call man_entity_forall_matching
    ret