;;------------------------------------------------LICENSE NOTICE------------------------------------------------------------------------------
;;  This file is part of Amstrad CPC videogame "Zombie Disco"
;;  Copyright (C) 2022 PandemixStudio / Angel Robles Gonzalez/ David Costa Blanco / Christian Jose Mira Castello
;;  Copyright (C) 2020 ronaldo / Fremos / Cheesetea / ByteRealms (@FranGallegoBR)
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;------------------------------------------------LICENSE NOTICE-------------------------------------------------------------------------
.module PHYSICS
.include "cpctelera.h.s"
.include "sys/physics.h.s"
.include "man/game.h.s"
.include "man/entity.h.s"
.include "man/animations.h.s"
.include "man/entity_templates.h.s"

;.include "sprites/zombie.h.s"
;.include "sprites/player.h.s"
.include "sprites/bullet.h.s"

VX_MAX = 1
VY_MAX = 2

UPDATES_ENEMY = 3
updates_enemy: .db 0


movement_left:
	ld a, #PLAYER_DIR_LEFT	
	ld (m_player_direccion), a
	call man_anim_change_sprite_anim
	
	ld e_vx(ix),#-VX_MAX
ret

movement_right:
	ld a, #PLAYER_DIR_RIGHT
	ld (m_player_direccion), a
	call man_anim_change_sprite_anim
	ld e_vx(ix),#VX_MAX
ret

movement_up:
	ld a, #PLAYER_DIR_UP
	ld (m_player_direccion), a
	call man_anim_change_sprite_anim
	ld e_vy(ix),#-VY_MAX
	jr z, endif_playerupdateArriba
ret

movement_down:
	ld a, #PLAYER_DIR_DOWN
	ld (m_player_direccion), a
	call man_anim_change_sprite_anim
	ld e_vy(ix),#VY_MAX
ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Comprueba los inputs del jugador y las colisiones
;;Input: ix - la entidad del jugador
;;
sys_physics_input_player::
	;call cpct_scanKeyboard_asm

	;Velociadad por defecto
	ld e_vx(ix), #0
	ld e_vy(ix), #0
	
	;Colision izq
	ld  a, e_x(ix)
	cp #4
	jr z, endif_playerupdateIzq
		
			ld   hl, #Key_O
			call cpct_isKeyPressed_asm
			jr z, check_joystickLeft
			;izquierda pulsada
				call movement_left
				jr endif_playerupdateIzq

			check_joystickLeft:
			;;Check joystick
			ld   hl, #Joy0_Left
			call cpct_isKeyPressed_asm
			jr z, endif_playerupdateIzq
			;izquierda pulsada
				call movement_left
				
	endif_playerupdateIzq:

	;Colision dere
	ld  a, e_x(ix)
	sub #(80 - SPR_PLAYER_0_W-4)
	jr nc, endif_playerupdateDere

			ld   hl, #Key_P
			call cpct_isKeyPressed_asm
			jr z, check_joystickRight
				; Derecha pulsada
				call movement_right
				jr endif_playerupdateDere

			check_joystickRight:
			;;Check joystick
			ld   hl, #Joy0_Right
			call cpct_isKeyPressed_asm
			jr z, endif_playerupdateDere
				; Derecha pulsada
				call movement_right
	endif_playerupdateDere:

	;Colision arriba
	ld  a, e_y(ix)
	cp #18
	jr z, endif_playerupdateArriba
			ld   hl, #Key_Q
			call cpct_isKeyPressed_asm
			jr z, check_joystickUp
				; Arriba pulsado
				call movement_up
				jr endif_playerupdateArriba
			check_joystickUp:
			;;Check joystick
			ld   hl, #Joy0_Up
			call cpct_isKeyPressed_asm
			jr z, endif_playerupdateArriba
				; Derecha pulsada
				call movement_up
	endif_playerupdateArriba:

	;Colision abajo
	ld  a, e_y(ix)
	sub #(150 - SPR_PLAYER_0_H)
	jr nc, endif_playerupdateAbajo

			ld   hl, #Key_A
			call cpct_isKeyPressed_asm
			jr z, check_joystickDown
				; Abajo pulsada
				call movement_down
				jr endif_playerupdateAbajo
			check_joystickDown:
			;;Check joystick
			ld   hl, #Joy0_Down
			call cpct_isKeyPressed_asm
			jr z, endif_playerupdateAbajo
				; Abajo pulsada
				call movement_down
	endif_playerupdateAbajo:

	;;Update shot cd
	call man_game_shot_cd

	ld   hl, #Key_Space
	call cpct_isKeyPressed_asm
	jr z, endif_playerupdateshot
		; Abajo pulsada
		call man_game_entity_shot
	endif_playerupdateshot:

	ld   hl, #Joy0_Fire1
	call cpct_isKeyPressed_asm
	jr z, endif_playerupdateshotJoystick
		; Abajo pulsada
		call man_game_entity_shot
	endif_playerupdateshotJoystick:

ret



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Update one entity from pointer
;;Input:
;;  -ix: entity pointer
sys_physics_update_one_entity:
	ld a,#e_cmp_input
	and e_cmp(ix)
	jr z,no_check_input
		call sys_physics_input_player
	no_check_input:
		call sys_physics_move
    ret

sys_physics_move::
	ld a, e_x(ix)
	add e_vx(ix)
	ld e_x(ix), a
	ld a, e_y(ix)
	add e_vy(ix)
	ld e_y(ix), a
ret
sys_physics_undo_move::
	ld a, (direccCollision)
	and #1
	cp #1
	jr nz, next_move
	ld a, e_x(ix)
	sub e_vx(ix)
	ld e_x(ix), a

	next_move:
	ld a, (direccCollision)
	and #2
	cp #2
	ret nz
	ld a, e_y(ix)
	sub e_vy(ix)
	ld e_y(ix), a

ret

sys_physics_update:
	ld hl, 	#sys_physics_update_one_entity
	ld  d,	#e_cmp_movable
	call man_entity_forall_matching
	ret

